@@ -22,14 +22,14 @@ STOP_WITHIN_SEQ2 = 8
 # Use this to get regular semiglobal alignment
 # (all gaps in the beginning or end are free)
 SEMIGLOBAL = (
-    START_WITHIN_SEQ1 | START_WITHIN_SEQ2 | 
+    START_WITHIN_SEQ1 | START_WITHIN_SEQ2 |
     STOP_WITHIN_SEQ1 | STOP_WITHIN_SEQ2)
 
 def compare_suffixes(
         suffix_ref, suffix_query, wildcard_ref=False, wildcard_query=False):
     """Find out whether one string is the suffix of the other one, allowing
     mismatches. Used to find an anchored 3' adapter when no indels are allowed.
-    
+
     Args:
         suffix_ref, suffix_query: The suffices to compare.
         wildcard_ref, wildcard_query: Whether wildcards are valid in either of
@@ -50,7 +50,7 @@ def compare_suffixes(
 
 class Match(object):
     """An alignment match.
-    
+
     Args:
         astart: Starting position of the match within the adapter.
         astop: Ending position of the match within the adapter.
@@ -66,7 +66,7 @@ class Match(object):
     __slots__ = [
         'astart', 'astop', 'rstart', 'rstop', 'matches', 'errors', 'front',
         'adapter', 'read', 'length']
-    
+
     def __init__(
             self, astart, astop, rstart, rstop, matches, errors,
             front=None, adapter=None, read=None):
@@ -82,8 +82,10 @@ class Match(object):
         # Number of aligned characters in the adapter. If there are indels,
         # this may be different from the number of characters in the read.
         self.length = self.astop - self.astart
-        assert self.length > 0
-        assert self.length - self.errors > 0
+        if self.length <= 0:
+            raise ValueError('Match length must be >= 0')
+        if self.length - self.errors <= 0:
+            raise ValueError('A Match requires at least one matching position.')
         # TODO: this assertion may not always hold now that we use both error
         # rates and probabilities
         #if self.adapter:
@@ -95,22 +97,22 @@ class Match(object):
             'errors={5})').format(
                 self.astart, self.astop, self.rstart, self.rstop, self.matches,
                 self.errors)
-    
+
     def copy(self):
         """Create a copy of this Match.
         """
         return Match(
             self.astart, self.astop, self.rstart, self.rstop, self.matches,
             self.errors, self.front, self.adapter, self.read)
-    
+
     def _guess_is_front(self):
         """Return whether this is guessed to be a front adapter.
-        
+
         The match is assumed to be a front adapter when the first base of
         the read is involved in the alignment to the adapter.
         """
         return self.rstart == 0
-    
+
     def wildcards(self, wildcard_char='N'):
         """Return a string that contains, for each wildcard character,
         the character that it matches. For example, if the adapter
@@ -136,7 +138,7 @@ class Match(object):
             return self.read.sequence[:self.rstart]
         else:
             return self.read.sequence[self.rstop:]
-    
+
     # TODO: write test
     def get_info_record(self):
         """Returns a :class:`MatchInfo`, which contains information about the
@@ -197,20 +199,34 @@ MatchInfo = namedtuple("MatchInfo", (
 #    http://www.isical.ac.in/~bioinfo_miu/FOGSAA.7z
 # 5. EDLIB: edit distance-based alignment
 #    https://github.com/Martinsos/edlib
-# 6. Phred-adjusted ML for error probability: 
+# 6. Phred-adjusted ML for error probability:
 # https://biosails.github.io/pheniqs/glossary.html#phred_adjusted_maximum_likelihood_decoding
 # 7. Adaptive banded alignment: https://github.com/ocxtal/libgaba
 # Also think about different sequence encodings that might enable faster alignment
 # https://github.com/hammerlab/kerseq/blob/master/kerseq/sequence_encoding.py
+# 8. https://github.com/yamada-kd/nepal
+# 9. The SeqAn C++ library implements several alignment algorithms:
+# http://www.sciencedirect.com/science/article/pii/S0168165617315420
+# 10. Could we treat paired end read + adapter alignment as an MSA problem?
+# 11. Look at alignment-free tools for pairwise sequence comparison:
+# * http://www.combio.pl/alfree/tools/
+# * http://www.combio.pl/alfree
+# * http://bioinformatics.org.au/tools/decaf+py/
+# 12. https://github.com/sdu-hpcl/BGSA
+# 13. Train a NN to approximate the pairwise alignment distance
+# https://academic.oup.com/bioinformatics/advance-article-abstract/doi/10.1093/bioinformatics/bty887/5140215?redirectedFrom=fulltext
+# 14. Nucl2vec: https://github.com/prakharg24/Nucl2vec (local alignment only - could
+# it be adapted to semi-global?)
+# * Can re-implement in SpaCy? https://spacy.io/usage/vectors-similarity
 
 class InsertAligner(object):
     """Implementation of an insert matching algorithm.
-    
+
     If the inserts align, the overhangs are searched for the adapter sequences.
     Otherwise, each read is search for its adapter separately.
-    
+
     This only works with paired-end reads with 3' adapters.
-    
+
     Args:
         adapter1, adapter2: read1, read2 adapters.
         match_probability: Callable that calculates random match probability
@@ -237,7 +253,8 @@ class InsertAligner(object):
             insert_max_rmp=1E-6, adapter_max_rmp=0.001,
             min_insert_overlap=1, max_insert_mismatch_frac=0.2,
             min_adapter_overlap=1, max_adapter_mismatch_frac=0.2,
-            adapter_check_cutoff=9, base_probs=None):
+            adapter_check_cutoff=9, base_probs=None,
+            adapter_wildcards=True, read_wildcards=False):
         self.adapter1 = adapter1
         self.adapter1_len = len(adapter1)
         self.adapter2 = adapter2
@@ -252,76 +269,85 @@ class InsertAligner(object):
         self.adapter_check_cutoff = adapter_check_cutoff
         self.base_probs = base_probs or dict(
             match_prob=0.25, mismatch_prob=0.75)
+        self.adapter_wildcards = adapter_wildcards
+        self.read_wildcards = read_wildcards
         self.aligner = MultiAligner(
             max_insert_mismatch_frac,
             START_WITHIN_SEQ1 | STOP_WITHIN_SEQ2,
             min_insert_overlap)
-    
+
     def match_insert(self, seq1, seq2):
         """Use cutadapt aligner for insert and adapter matching.
-        
+
         Args:
             seq1, seq2: Sequences to match.
-        
+
         Returns:
             A :class:`Match` object, or None if there is no match.
         """
-        len1 = len(seq1)
-        len2 = len(seq2)
-        seq_len = min(len1, len2)
-        if len1 > len2:
-            seq1 = seq1[:len2]
-        elif len2 > len1:
-            seq2 = seq1[:len1]
+        seq_len1 = len(seq1)
+        seq_len2 = len(seq2)
+        seq_len = min(seq_len1, seq_len2)
+        if seq_len1 > seq_len2:
+            seq1 = seq1[:seq_len2]
+        elif seq_len2 > seq_len1:
+            seq2 = seq2[:seq_len1]
 
         seq2_rc = reverse_complement(seq2)
-        
-        def _match(insert_match, offset, insert_match_size, prob): # pylint disable=unused-argument
-            if offset < self.min_adapter_overlap:
+
+        def _match(_insert_match, _offset, _insert_match_size, _):
+            if _offset < self.min_adapter_overlap:
                 # The reads are mostly overlapping, to the point where
                 # there's not enough overhang to do a confident adapter
                 # match. We return just the insert match to signal that
                 # error correction can be done even though no adapter
                 # trimming is required.
-                return (insert_match, None, None)
-            
+                return (_insert_match, None, None)
+
             # TODO: this is very sensitive to the exact correct choice of
             # adapter. For example, if you specifiy GATCGGAA... and the correct
             # adapter is AGATCGGAA..., the prefixes will not match exactly and
             # the alignment will fail. We need to use a comparison that is a bit
             # more forgiving.
-            
-            a1_match = compare_prefixes(seq1[insert_match_size:], self.adapter1)
-            a2_match = compare_prefixes(seq2[insert_match_size:], self.adapter2)
-            adapter_len = min(offset, self.adapter1_len, self.adapter2_len)
-            max_adapter_mismatches = round(
-                adapter_len * self.max_adapter_mismatch_frac)
+
+            def _adapter_match(insert_seq, adapter_seq, adapter_len):
+                amatch = compare_prefixes(
+                    insert_seq[_insert_match_size:], adapter_seq,
+                    wildcard_ref=self.adapter_wildcards,
+                    wildcard_query=self.read_wildcards)
+                alen = min(_offset, adapter_len)
+                return amatch, alen, round(alen * self.max_adapter_mismatch_frac)
+
+            a1_match, a1_length, a1_max_mismatches = _adapter_match(
+                seq1, self.adapter1, self.adapter1_len)
+            a2_match, a2_length, a2_max_mismatches = _adapter_match(
+                seq2, self.adapter2, self.adapter2_len)
+
             if (
-                    a1_match[5] > max_adapter_mismatches and
-                    a2_match[5] > max_adapter_mismatches):
-                return None
-            
-            a1_prob = self.match_probability(a1_match[4], adapter_len)
-            a2_prob = self.match_probability(a2_match[4], adapter_len)
-            if (
-                    (adapter_len > self.adapter_check_cutoff) and
-                    ((a1_prob * a2_prob) > self.adapter_max_rmp)):
+                    a1_match[5] > a1_max_mismatches and
+                    a2_match[5] > a2_max_mismatches):
                 return None
 
-            adapter_len1 = min(self.adapter1_len, len1 - insert_match_size)
-            adapter_len2 = min(self.adapter2_len, len2 - insert_match_size)
-            best_adapter_matches, best_adapter_mismatches = (
-                a1_match if a1_prob < a2_prob else a2_match)[4:6]
-            
+            if min(a1_length, a2_length) > self.adapter_check_cutoff:
+                a1_prob = self.match_probability(a1_match[4], a1_length)
+                a2_prob = self.match_probability(a2_match[4], a2_length)
+                if (a1_prob * a2_prob) > self.adapter_max_rmp:
+                    return None
+
+            mismatches = min(a1_match[5], a2_match[5])
+
+            def _create_match(alen, slen):
+                alen = min(alen, slen - _insert_match_size)
+                _mismatches = min(alen, mismatches)
+                _matches = alen - _mismatches
+                return Match(0, alen, _insert_match_size, slen, _matches, _mismatches)
+
             return (
-                insert_match,
-                Match(
-                    0, adapter_len1, insert_match_size, len1,
-                    best_adapter_matches, best_adapter_mismatches),
-                Match(
-                    0, adapter_len2, insert_match_size, len2,
-                    best_adapter_matches, best_adapter_mismatches))
-        
+                _insert_match,
+                _create_match(a1_length, seq_len1),
+                _create_match(a2_length, seq_len2)
+            )
+
         # # This is the old way of doing things, where we use the built-in
         # # Aligner to do a single match.
         # aligner = Aligner(
@@ -345,14 +371,14 @@ class InsertAligner(object):
         #     return None
         #
         # return _match(insert_match, offset, insert_match_size, prob)
-        
+
         # Use an aligner that returns all matches that satisfy the
         # overlap and error rate thresholds. We sort by matches and
         # then mismatches, and then check each in turn until we find
         # one with an adapter match (if any).
-        
+
         insert_matches = self.aligner.locate(seq2_rc, seq1)
-        
+
         if insert_matches:
             # Filter by random-match probability
             filtered_matches = []
@@ -362,7 +388,7 @@ class InsertAligner(object):
                 prob = self.match_probability(insert_match[4], insert_match_size, **self.base_probs)
                 if prob <= self.insert_max_rmp:
                     filtered_matches.append((insert_match, offset, insert_match_size, prob))
-            
+
             if filtered_matches:
                 if len(filtered_matches) == 1:
                     return _match(*filtered_matches[0])
@@ -370,11 +396,11 @@ class InsertAligner(object):
                     # Test matches in order of random-match probability.
                     # TODO: compare against sorting by length (which is how
                     # SeqPurge essentially does it).
-                    #filtered_matches.sort(key=lambda x: x[2], reverse=True)
+                    # filtered_matches.sort(key=lambda x: x[2], reverse=True)
                     filtered_matches.sort(key=lambda x: x[3])
                     for match_args in filtered_matches:
                         match = _match(*match_args)
                         if match:
                             return match
-            
+
             return None
