@@ -338,7 +338,7 @@ def print_trim_report(summary, outfile):
                     indent=(INDENT, ''), pct=True)
         else:
             _print(
-                "Reads with adapter:",
+                "Reads with adapters:",
                 adapter_cutter['records_with_adapters'][0],
                 adapter_cutter['fraction_records_with_adapters'][0],
                 pct=True)
@@ -452,7 +452,9 @@ def print_adapter_report(adapters, outfile, paired, total_records, max_width):
                 prev, adapter_length, int(error_rate * adapter_length)))
         _print()
     
-    def print_histogram(data, adapter_length, num_reads, error_rate, errors, match_probabilities):
+    def print_histogram(
+            data, adapter_length, num_reads, error_rate, errors, 
+            match_probabilities):
         """Print a histogram. Also, print the no. of reads expected to be
         trimmed by chance (assuming a uniform distribution of nucleotides in
         the reads).
@@ -527,11 +529,11 @@ def print_adapter_report(adapters, outfile, paired, total_records, max_width):
                 warnbase = base_label
         if total >= 20 and warnbase is not None:
             _print('WARNING:')
-            _print(INDENTED.wrap(
+            _print("\n".join(INDENTED.wrap(
                 'The adapter is preceded by "{0}" extremely often. The '
                 'provided adapter sequence may be incomplete. To fix the '
                 'problem, add "{0}" to the beginning of the adapter '
-                'sequence.'.format(warnbase)))
+                'sequence.'.format(warnbase))))
             _print()
             return True
         _print()
@@ -578,7 +580,7 @@ def print_adapter_report(adapters, outfile, paired, total_records, max_width):
             _print()
             
             if adapter["total"] == 0:
-                return
+                continue
             
             if where_name == "anywhere":
                 _print(
@@ -637,9 +639,9 @@ def print_adapter_report(adapters, outfile, paired, total_records, max_width):
     
     if warning:
         _print('WARNING:')
-        _print(INDENTED.wrap(
+        _print("\n".join(INDENTED.wrap(
             'One or more of your adapter sequences may be incomplete. '
-            'Please see the detailed output above.'))
+            'Please see the detailed output above.')))
 
 def print_pre_trim_report(summary, outfile):
     """Print pre-trimming stats.
@@ -707,6 +709,9 @@ def print_stats_report(data, outfile):
     
     def _print_histogram(title, hist1, hist2=None):
         _print_title(title, level=2)
+        if hist1 is None:
+            _print("No Data")
+            return
         if hist2:
             hist = (
                 (key, hist1.get(key, 0), hist2.get(key, 0))
@@ -718,6 +723,9 @@ def print_stats_report(data, outfile):
     
     def _print_base_histogram(title, hist, extra_width=4, index_name='Pos'):
         _print_title(title, level=2)
+        if hist is None:
+            _print("No Data")
+            return
         _print(
             index_name, *hist['columns'], header=True, extra_width=extra_width)
         for pos, row in hist['rows'].items():
@@ -728,6 +736,10 @@ def print_stats_report(data, outfile):
             _print(pos, *base_pcts, extra_width=extra_width)
     
     def _print_tile_histogram(title, hist):
+        if hist is None:
+            _print_title(title, level=2)
+            _print("No Data")
+            return
         ncol = len(hist['columns'])
         max_tile_width = max(
             4, len(str(math.ceil(data['read1']['counts'] / ncol)))) + 1
@@ -738,12 +750,15 @@ def print_stats_report(data, outfile):
         """Print a histogram of position x tile, with values as the median
         base quality.
         """
+        _print_title(title, level=2)
+        if hist is None:
+            _print("No Data")
+            return
         quals = hist['columns']
         tiles = hist['columns2']
         ncol = len(tiles)
         max_tile_width = max(
             4, len(str(math.ceil(data['read1']['counts'] / ncol)))) + 1
-        _print_title(title, level=2)
         _print('Pos', *tiles, header=True, extra_width=max_tile_width)
         for pos, tiles in hist['rows'].items():
             # compute the weighted median for each tile at each position
