@@ -9,7 +9,7 @@ import sys
 from atropos import AtroposError
 from atropos.io import STDOUT, xopen
 from atropos.io.compression import splitext_compressed
-from atropos.util import Summarizable, truncate_string
+from atropos.util import Summarizable, truncate_string, ALPHABETS
 
 READ1 = 1
 READ2 = 2
@@ -50,11 +50,14 @@ class SequenceReaderBase(Summarizable): # pylint: disable=no-member
 
 class SequenceReader(SequenceReaderBase):
     """Read possibly compressed files containing sequences.
-    
+
     Args:
         file is a path or a file-like object. In both cases, the file may
             be compressed (.gz, .bz2, .xz).
         mode: The file open mode.
+        quality_base: The minimum quality value.
+        alphabet: The alphabet to use to validate sequences. If None, no
+            validation is done.
     """
     delivers_qualities = False
     has_qualfile = False
@@ -62,9 +65,10 @@ class SequenceReader(SequenceReaderBase):
     interleaved = False
     input_read = SINGLE
     _close_on_exit = False
-    
-    def __init__(self, path, mode='r', quality_base=None):
+
+    def __init__(self, path, mode='r', quality_base=None, alphabet=None):
         self.quality_base = quality_base
+        self.alphabet = alphabet
         if isinstance(path, str):
             self.name = path
             self._file = xopen(path, mode)
@@ -76,23 +80,23 @@ class SequenceReader(SequenceReaderBase):
                 # TODO: generate random unique name?
                 self.name = path.__class__
             self._file = path
-    
+
     @property
     def input_names(self):
         return (self.name, None)
-    
+
     def close(self):
         """Close the underlying file.
         """
         if self._close_on_exit and self._file is not None:
             self._file.close()
             self._file = None
-    
+
     def __enter__(self):
         if self._file is None:
             raise ValueError("I/O operation on closed SequenceReader")
         return self
-    
+
     def __exit__(self, *args):
         self.close()
 
@@ -107,7 +111,7 @@ class ColorspaceSequence(Sequence):
     def __init__(
             self, name, sequence, qualities, primer=None, name2='',
             original_length=None, match=None, match_info=None, clipped=None,
-            insert_overlap=False, merged=False, corrected=0):
+            insert_overlap=False, merged=False, corrected=0, alphabet=None):
         # In colorspace, the first character is the last nucleotide of the
         # primer base and the second character encodes the transition from the
         # primer base to the first real base of the read.
@@ -125,12 +129,14 @@ class ColorspaceSequence(Sequence):
                     rname, len(qualities), len(sequence), self.primer))
         super().__init__(
             name, sequence, qualities, name2, original_length, match,
-            match_info, clipped, insert_overlap, merged, corrected)
+            match_info, clipped, insert_overlap, merged, corrected,
+            alphabet=alphabet)
+        # TODO: use 'alphabet' here
         if not self.primer in ('A', 'C', 'G', 'T'):
             raise FormatError(
                 "Primer base is {0!r} in read {1!r}, but it should be one of "
                 "A, C, G, T.".format(self.primer, truncate_string(name)))
-    
+
     def __repr__(self):
         fmt_str = \
             '<ColorspaceSequence(name={0!r}, primer={1!r}, sequence={2!r}{3})>'
@@ -140,7 +146,7 @@ class ColorspaceSequence(Sequence):
         return fmt_str.format(
             truncate_string(self.name), self.primer,
             truncate_string(self.sequence), qstr)
-    
+
     def __getitem__(self, key):
         return self.__class__(
             self.name,
@@ -159,12 +165,14 @@ class ColorspaceSequence(Sequence):
 class SraSequenceReader(SequenceReader):
     delivers_qualities = True
     file_format = "fastq"
-    
-    def __init__(self, reader, quality_base=None, sequence_class=Sequence):
-        super().__init__(reader, quality_base=quality_base)
+
+    def __init__(
+            self, reader, quality_base=None, sequence_class=Sequence,
+            alphabet=None):
+        super().__init__(reader, quality_base=quality_base, alphabet=alphabet)
         self.input_read = PAIRED if reader.paired else SINGLE
         self.sequence_class = sequence_class
-    
+
     def __iter__(self):
         if self.input_read == PAIRED:
             for read in self._file:
@@ -172,10 +180,10 @@ class SraSequenceReader(SequenceReader):
         else:
             for read in self._file:
                 yield self._as_sequence(read[0])
-    
+
     def _as_sequence(self, frag):
-        return self.sequence_class(*frag)
-    
+        return self.sequence_class(*frag, alphabet=self.alphabet)
+
     def close(self):
         self._file.finish()
 
@@ -183,11 +191,12 @@ class SraColorspaceSequenceReader(SraSequenceReader):
     """Reads colorspace sequences from an SRA accession.
     """
     colorspace = True
-    
-    def __init__(self, reader, quality_base=33):
+
+    def __init__(self, reader, quality_base=33, alphabet=None):
         super().__init__(
-            reader, quality_base=quality_base, 
-            sequence_class=ColorspaceSequence)
+            reader, quality_base=quality_base,
+            sequence_class=ColorspaceSequence,
+            alphabet=alphabet)
 
 class FileWithPrependedLine(object):
     """A file-like object that allows to "prepend" a single line to an already
@@ -196,7 +205,7 @@ class FileWithPrependedLine(object):
     of autodetecting input from a stream: As soon as the first line has been
     read, we know the file type, but also that line is "gone" and unavailable
     for further processing.
-    
+
     Args:
         file: An already opened file-like object.
         line: A single string (newline will be appended if not included).
@@ -206,16 +215,16 @@ class FileWithPrependedLine(object):
             line += '\n'
         self.first_line = line
         self._file = file
-    
+
     @property
     def name(self):
         return self._file.name
-    
+
     def __iter__(self):
         yield self.first_line
         for line in self._file:
             yield line
-    
+
     def close(self):
         """Close the underlying file.
         """
@@ -223,7 +232,7 @@ class FileWithPrependedLine(object):
 
 class FastaReader(SequenceReader):
     """Reader for FASTA files.
-    
+
     Args:
         path: A path or a file-like object. In both cases, the file may
             be compressed (.gz, .bz2, .xz).
@@ -231,12 +240,14 @@ class FastaReader(SequenceReader):
         sequence_class: The class to use when creating new sequence objects.
     """
     file_format = "FASTA"
-    
-    def __init__(self, path, keep_linebreaks=False, sequence_class=Sequence):
-        super().__init__(path)
+
+    def __init__(
+            self, path, keep_linebreaks=False, sequence_class=Sequence,
+            alphabet=None):
+        super().__init__(path, alphabet=alphabet)
         self.sequence_class = sequence_class
         self._delimiter = '\n' if keep_linebreaks else ''
-    
+
     def __iter__(self):
         """Read next entry from the file (single entry at a time).
         """
@@ -250,7 +261,8 @@ class FastaReader(SequenceReader):
             if line and line[0] == '>':
                 if name is not None:
                     yield self.sequence_class(
-                        name, self._delimiter.join(seq), None)
+                        name, self._delimiter.join(seq), None,
+                        alphabet=self.alphabet)
                 name = line[1:]
                 seq = []
             elif line and line[0] == '#':
@@ -261,45 +273,50 @@ class FastaReader(SequenceReader):
                 raise FormatError(
                     "At line {0}: Expected '>' at beginning of FASTA record, "
                     "but got {1!r}.".format(i+1, truncate_string(line)))
-        
+
         if name is not None:
-            yield self.sequence_class(name, self._delimiter.join(seq), None)
+            yield self.sequence_class(
+                name, self._delimiter.join(seq), None,
+                alphabet=self.alphabet)
 
 class ColorspaceFastaReader(FastaReader):
     """Reads colorspace sequences from a FASTA.
-    
+
     Args:
         path: The file to read.
         keep_linebreaks: Whether to keep linebreaks in wrapped sequences.
     """
     colorspace = True
-    
-    def __init__(self, path, keep_linebreaks=False):
+
+    def __init__(self, path, keep_linebreaks=False, alphabet=None):
         super().__init__(
-            path, keep_linebreaks, sequence_class=ColorspaceSequence)
+            path, keep_linebreaks, sequence_class=ColorspaceSequence,
+            alphabet=alphabet)
 
 class ColorspaceFastqReader(FastqReader):
     """Reads colorspace sequences from a FASTQ.
     """
     colorspace = True
-    
-    def __init__(self, path, quality_base=33):
+
+    def __init__(self, path, quality_base=33, alphabet=None):
         super().__init__(
-            path, quality_base=quality_base, sequence_class=ColorspaceSequence)
+            path, quality_base=quality_base, sequence_class=ColorspaceSequence,
+            alphabet=alphabet)
 
 class SRAColorspaceFastqReader(FastqReader):
     """Reads SRA-formatted colorspace sequences from a FASTQ.
     """
     colorspace = True
-    
-    def __init__(self, path, quality_base=33):
+
+    def __init__(self, path, quality_base=33, alphabet=None):
         super().__init__(
             path, quality_base=quality_base,
-            sequence_class=sra_colorspace_sequence)
+            sequence_class=sra_colorspace_sequence,
+            alphabet=alphabet)
 
 class FastaQualReader(SequenceReaderBase):
     """Reader for reads that are stored in .(CS)FASTA and .QUAL files.
-    
+
     Args:
         fastafile and qualfile are filenames or file-like objects.
             If a filename is used, then .gz files are recognized.
@@ -311,19 +328,20 @@ class FastaQualReader(SequenceReaderBase):
     colorspace = False
     interleaved = False
     input_read = SINGLE
-    
+
     def __init__(
             self, fastafile, qualfile, quality_base=33,
-            sequence_class=Sequence):
+            sequence_class=Sequence, alphabet=None):
         self.fastareader = FastaReader(fastafile)
         self.qualreader = FastaReader(qualfile, keep_linebreaks=True)
         self.quality_base = quality_base
         self.sequence_class = sequence_class
-    
+        self.alphabet = alphabet
+
     @property
     def input_names(self):
         return ((self.fastareader.name, self.qualreader.name), None)
-    
+
     def __iter__(self):
         """Yield Sequence objects.
         """
@@ -346,39 +364,40 @@ class FastaQualReader(SequenceReaderBase):
                     "value {1}".format(fastaread.name, err))
             assert fastaread.name == qualread.name
             yield self.sequence_class(
-                fastaread.name, fastaread.sequence, qualities)
-    
+                fastaread.name, fastaread.sequence, qualities,
+                alphabet=self.alphabet)
+
     def close(self):
         """Close the underlying files.
         """
         self.fastareader.close()
         self.qualreader.close()
-    
+
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *args):
         self.close()
 
 class ColorspaceFastaQualReader(FastaQualReader):
     """Reads sequences and qualities from separate files and returns
     :class:`ColorspaceSequence`s.
-    
+
     Args:
         fastafile, qualfile: FASTA files that contain the sequences and
             qualities, respectively.
     """
     colorspace = True
-    
-    def __init__(self, fastafile, qualfile, quality_base=33):
+
+    def __init__(self, fastafile, qualfile, quality_base=33, alphabet=None):
         super().__init__(
             fastafile, qualfile, quality_base=quality_base,
-            sequence_class=ColorspaceSequence)
+            sequence_class=ColorspaceSequence, alphabet=alphabet)
 
 class PairedSequenceReader(SequenceReaderBase):
-    """Read paired-end reads from two files. Wraps two SequenceReader instances,
-    making sure that reads are properly paired.
-    
+    """Read paired-end reads from two files. Wraps two SequenceReader
+    instances, making sure that reads are properly paired.
+
     Args:
         file1, file2: The pair of files.
         colorspace: Whether the sequences are in colorspace.
@@ -386,26 +405,26 @@ class PairedSequenceReader(SequenceReaderBase):
     """
     input_read = PAIRED
     interleaved = False
-    
+
     def __init__(
             self, file1, file2, quality_base=33, colorspace=False,
-            file_format=None):
+            file_format=None, alphabet=None):
         self.reader1 = open_reader(
             file1, colorspace=colorspace, quality_base=quality_base,
-            file_format=file_format)
+            file_format=file_format, alphabet=alphabet)
         self.reader2 = open_reader(
             file2, colorspace=colorspace, quality_base=quality_base,
-            file_format=file_format)
-    
+            file_format=file_format, alphabet=alphabet)
+
     @property
     def input_names(self):
         return (
-            self.reader1.input_names[0], 
+            self.reader1.input_names[0],
             self.reader2.input_names[0])
-    
+
     def __getattr__(self, name):
         return getattr(self.reader1, name)
-    
+
     def __iter__(self):
         """Iterate over the paired reads. Each item is a pair of Sequence
         objects.
@@ -437,22 +456,22 @@ class PairedSequenceReader(SequenceReaderBase):
                     "does not match '{1}' in file 2.".format(
                         read1.name, read2.name))
             yield (read1, read2)
-    
+
     def close(self):
         """Close the underlying files.
         """
         self.reader1.close()
         self.reader2.close()
-    
+
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *args):
         self.close()
 
 class InterleavedSequenceReader(SequenceReaderBase):
     """Read paired-end reads from an interleaved FASTQ file.
-    
+
     Args:
         path: The interleaved FASTQ file.
         colorspace: Whether the sequences are in colorspace.
@@ -460,16 +479,17 @@ class InterleavedSequenceReader(SequenceReaderBase):
     """
     input_read = PAIRED
     interleaved = True
-    
+
     def __init__(
-            self, path, quality_base=33, colorspace=False, file_format=None):
+            self, path, quality_base=33, colorspace=False, file_format=None,
+            alphabet=None):
         self.reader = open_reader(
             path, quality_base=quality_base, colorspace=colorspace,
-            file_format=file_format)
-    
+            file_format=file_format, alphabet=alphabet)
+
     def __getattr__(self, name):
         return getattr(self.reader, name)
-    
+
     def __iter__(self):
         # Avoid usage of zip() below since it will consume one item too many.
         itr = iter(self.reader)
@@ -485,7 +505,7 @@ class InterleavedSequenceReader(SequenceReaderBase):
                     "Reads are improperly paired. Name {0!r} (first) does not "
                     "match {1!r} (second).".format(read1.name, read2.name))
             yield (read1, read2)
-    
+
     def close(self):
         """Close the underlying reader.
         """
@@ -502,7 +522,7 @@ class InterleavedSequenceReader(SequenceReaderBase):
 class SAMReader(SequenceReaderBase):
     """Reader for SAM/BAM files. Paired-end files must be name-sorted. Does
     not support secondary/supplementary reads. This is an abstract class.
-    
+
     Args:
         path: A filename or a file-like object. If a filename, then .gz files
             are supported.
@@ -513,8 +533,10 @@ class SAMReader(SequenceReaderBase):
     interleaved = False
     has_qualfile = False
     colorspace = False
-    
-    def __init__(self, path, quality_base=33, sequence_class=Sequence):
+
+    def __init__(
+            self, path, quality_base=33, sequence_class=Sequence,
+            alphabet=None, pysam_kwargs=None):
         self._close_on_exit = False
         if isinstance(path, str):
             path = xopen(path, 'rb')
@@ -526,44 +548,47 @@ class SAMReader(SequenceReaderBase):
         self._file = path
         self.quality_base = quality_base
         self.sequence_class = sequence_class
-    
+        self.alphabet = alphabet
+        self.pysam_kwargs = pysam_kwargs or dict(check_sq=False)
+
     @property
     def input_names(self):
         return (self.name, None)
-    
+
     def __iter__(self):
         import pysam
-        return self._iter(pysam.AlignmentFile(self._file))
-    
+        return self._iter(pysam.AlignmentFile(self._file, **self.pysam_kwargs))
+
     def _iter(self, sam):
         """Create an iterator over records in the SAM/BAM file.
         """
         raise NotImplementedError()
-    
+
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *args):
         self.close()
-    
+
     def close(self):
         """Close the underling AlignmentFile.
         """
         if self._close_on_exit and self._file is not None:
             self._file.close()
             self._file = None
-        
+
     def _as_sequence(self, read):
         return self.sequence_class(
             read.query_name,
             read.query_sequence,
-            ''.join(chr(33 + q) for q in read.query_qualities))
+            ''.join(chr(33 + q) for q in read.query_qualities),
+            alphabet=self.alphabet)
 
 class SingleEndSAMReader(SAMReader):
     """Reader for single-end SAM/BAM files.
     """
     input_read = SINGLE
-    
+
     def _iter(self, sam):
         for read in sam:
             yield self._as_sequence(read)
@@ -573,7 +598,7 @@ class Read1SingleEndSAMReader(SAMReader):
     only the first read from each pair.
     """
     input_read = READ1
-    
+
     def _iter(self, sam):
         for read in sam:
             if read.is_read1:
@@ -584,7 +609,7 @@ class Read2SingleEndSAMReader(SAMReader):
     only the second read from each pair.
     """
     input_read = READ2
-    
+
     def _iter(self, sam):
         for read in sam:
             if read.is_read2:
@@ -595,7 +620,7 @@ class PairedEndSAMReader(SAMReader):
     """
     input_read = PAIRED
     interleaved = True
-    
+
     def _iter(self, sam):
         for reads in zip(sam, sam):
             if reads[0].query_name != reads[1].query_name:
@@ -605,24 +630,24 @@ class PairedEndSAMReader(SAMReader):
                     "name-sorted and does not contain any "
                     "secondary/supplementary alignments.",
                     reads[0].query_name, reads[1].query_name)
-            
+
             if reads[0].is_read1:
                 assert reads[1].is_read2
             else:
                 assert reads[1].is_read1
                 reads = (reads[1], reads[0])
-            
+
             yield tuple(self._as_sequence(r) for r in reads)
 
-class SequenceFileFormat(object):
+class SequenceFileFormat():
     """Base class for sequence formatters.
     """
     def format(self, read):
         """Format a Sequence as a string.
-        
+
         Args:
             read: The Sequence object.
-        
+
         Returns:
             A string representation of the sequence object in the sequence
             file format.
@@ -631,7 +656,7 @@ class SequenceFileFormat(object):
 
 class FastaFormat(SequenceFileFormat):
     """FASTA SequenceFileFormat.
-    
+
     Args:
         line_length: Max line length (in characters), or None. Determines
             whether and how lines are wrapped.
@@ -641,10 +666,10 @@ class FastaFormat(SequenceFileFormat):
         if line_length:
             from textwrap import TextWrapper
             self.text_wrapper = TextWrapper(width=line_length)
-    
+
     def format(self, read):
         return self.format_entry(read.name, read.sequence)
-    
+
     def format_entry(self, name, sequence):
         """Convert a sequence record to a string.
         """
@@ -664,7 +689,7 @@ class FastqFormat(SequenceFileFormat):
     def format(self, read):
         return self.format_entry(
             read.name, read.sequence, read.qualities, read.name2)
-    
+
     def format_entry(self, name, sequence, qualities, name2=""):
         """Convert a sequence record to a string.
         """
@@ -681,9 +706,9 @@ class ColorspaceFastqFormat(FastqFormat):
         return self.format_entry(
             read.name, read.primer + read.sequence, read.qualities)
 
-class SingleEndFormatter(object):
+class SingleEndFormatter():
     """Wrapper for a SequenceFileFormat for single-end data.
-    
+
     Args:
         seq_format: The SequenceFileFormat object.
         file1: The single-end file.
@@ -694,10 +719,10 @@ class SingleEndFormatter(object):
         self.written = 0
         self.read1_bp = 0
         self.read2_bp = 0
-    
+
     def format(self, result, read1, read2=None):
         """Format read(s) and add them to `result`.
-        
+
         Args:
             result: A dict mapping file names to lists of formatted reads.
             read1, read2: The reads to format.
@@ -705,7 +730,7 @@ class SingleEndFormatter(object):
         result[self.file1].append(self.seq_format.format(read1))
         self.written += 1
         self.read1_bp += len(read1)
-    
+
     @property
     def written_bp(self):
         """Tuple of base-pairs written (read1_bp, read2_bp).
@@ -730,7 +755,7 @@ class PairedEndFormatter(SingleEndFormatter):
     def __init__(self, seq_format, file1, file2):
         super(PairedEndFormatter, self).__init__(seq_format, file1)
         self.file2 = file2
-    
+
     def format(self, result, read1, read2):
         result[self.file1].append(self.seq_format.format(read1))
         result[self.file2].append(self.seq_format.format(read2))
@@ -738,11 +763,12 @@ class PairedEndFormatter(SingleEndFormatter):
         self.read1_bp += len(read1)
         self.read2_bp += len(read2)
 
-def sra_colorspace_sequence(name, sequence, qualities, name2):
+def sra_colorspace_sequence(name, sequence, qualities, name2, alphabet=None):
     """Factory for an SRA colorspace sequence (which has one quality value
     too many).
     """
-    return ColorspaceSequence(name, sequence, qualities[1:], name2=name2)
+    return ColorspaceSequence(
+        name, sequence, qualities[1:], name2=name2, alphabet=alphabet)
 
 def sequence_names_match(read1, read2):
     """Check whether the sequences read1 and read2 have identical names,
@@ -750,10 +776,10 @@ def sequence_names_match(read1, read2):
     end in '/1' and '/2'. Also, the fastq-dump tool (used for converting SRA
     files to FASTQ) appends a .1 and .2 to paired-end reads if option -I is
     used.
-    
+
     Args:
         read1, read2: The sequences to compare.
-    
+
     Returns:
         Whether the sequences are equal.
     """
@@ -777,13 +803,13 @@ def paired_to_read2(reader):
         yield read2
 
 def open_reader(
-        file1=None, file2=None, qualfile=None, quality_base=None, 
-        colorspace=False, file_format=None, interleaved=False, 
-        input_read=None):
+        file1=None, file2=None, qualfile=None, quality_base=None,
+        colorspace=False, file_format=None, interleaved=False,
+        input_read=None, alphabet=None):
     """Open sequence files in FASTA or FASTQ format for reading. This is
     a factory that returns an instance of one of the ...Reader
     classes also defined in this module.
-    
+
     Args:
         file1, file2, qualfile: Paths to regular or compressed files or
             file-like objects. Use file1 if data is single-end. If file2 is also
@@ -803,30 +829,37 @@ def open_reader(
         input_read: When file1 is a paired-end interleaved or SAM/BAM
             file, this specifies whether to only use the first or second read
             (1 or 2) or to use both reads (None).
+        alphabet: An Alphabet instance - the alphabet to use to validate
+            sequences.
     """
     if interleaved and (file2 is not None or qualfile is not None):
         raise ValueError(
             "When interleaved is set, file2 and qualfile must be None")
     if file2 is not None and qualfile is not None:
         raise ValueError("Setting both file2 and qualfile is not supported")
-    
+
+    if alphabet and isinstance(alphabet, str):
+        if not alphabet in ALPHABETS:
+            raise ValueError("Invalid alphabet {}".format(alphabet))
+        alphabet = ALPHABETS[alphabet]
+
     if file2 is not None:
         return PairedSequenceReader(
             file1, file2, quality_base=quality_base, colorspace=colorspace,
-            file_format=file_format)
-    
+            file_format=file_format, alphabet=alphabet)
+
     if qualfile is not None:
         if colorspace:
             # read from .(CS)FASTA/.QUAL
             return ColorspaceFastaQualReader(
-                file1, qualfile, quality_base=quality_base)
+                file1, qualfile, quality_base=quality_base, alphabet=alphabet)
         else:
             return FastaQualReader(
-                file1, qualfile, quality_base=quality_base)
-    
+                file1, qualfile, quality_base=quality_base, alphabet=alphabet)
+
     if file_format is None and file1 != STDOUT:
         file_format = guess_format_from_name(file1)
-    
+
     if file_format is None:
         if file1 == STDOUT:
             file1 = sys.stdin
@@ -841,7 +874,7 @@ def open_reader(
             # TODO: guess SAM/BAM from data
             file1 = FileWithPrependedLine(file1, line)
             break
-    
+
     if file_format is not None:
         file_format = file_format.lower()
         if file_format in ("sam", "bam"):
@@ -850,17 +883,21 @@ def open_reader(
                     "SAM/BAM format is not currently supported for colorspace "
                     "reads")
             if interleaved:
-                return PairedEndSAMReader(file1, quality_base=quality_base)
+                return PairedEndSAMReader(
+                    file1, quality_base=quality_base, alphabet=alphabet)
             elif input_read == READ1:
-                return Read1SingleEndSAMReader(file1, quality_base=quality_base)
+                return Read1SingleEndSAMReader(
+                    file1, quality_base=quality_base, alphabet=alphabet)
             elif input_read == READ2:
-                return Read2SingleEndSAMReader(file1, quality_base=quality_base)
+                return Read2SingleEndSAMReader(
+                    file1, quality_base=quality_base, alphabet=alphabet)
             else:
-                return SingleEndSAMReader(file1, quality_base=quality_base)
+                return SingleEndSAMReader(
+                    file1, quality_base=quality_base, alphabet=alphabet)
         elif interleaved:
             reader = InterleavedSequenceReader(
                 file1, quality_base=quality_base, colorspace=colorspace,
-                file_format=file_format)
+                file_format=file_format, alphabet=alphabet)
             if input_read == READ1:
                 return paired_to_read1(reader)
             elif input_read == READ2:
@@ -869,23 +906,29 @@ def open_reader(
                 return reader
         elif file_format == 'fasta':
             fasta_handler = ColorspaceFastaReader if colorspace else FastaReader
-            return fasta_handler(file1)
+            return fasta_handler(file1, alphabet=alphabet)
         elif file_format == 'fastq':
             fastq_handler = ColorspaceFastqReader if colorspace else FastqReader
-            return fastq_handler(file1, quality_base=quality_base)
+            return fastq_handler(
+                file1, quality_base=quality_base, alphabet=alphabet)
         elif file_format == 'sra-fastq' and colorspace:
-            return SRAColorspaceFastqReader(file1, quality_base=quality_base)
-    
+            return SRAColorspaceFastqReader(
+                file1, quality_base=quality_base, alphabet=alphabet)
+
     raise UnknownFileType(
         "File format {0!r} is unknown (expected 'sra-fastq' (only for "
-        "colorspace), 'fasta', 'fastq', 'sam', or 'bam').".format(file_format))
+        "colorspace), 'fasta', 'fastq', 'sam', or 'bam').".format(
+            file_format or '<Undetected>'
+        ))
 
-def sra_reader(reader, quality_base=None, colorspace=False, input_read=None):
+def sra_reader(
+        reader, quality_base=None, colorspace=False, input_read=None,
+        alphabet=None):
     """Wrap an existing SraReader. The reader must 1) have a 'paired' property,
     and 2) be iterable. Furthermore, each value yielded by the iterator must
     be a list of N tuples, where N = 2 if paired else 1, and where each tuple
     is (name, sequence, qualities).
-    
+
     Args:
         reader: An existing reader.
         quality_base: Base for quality values.
@@ -894,15 +937,19 @@ def sra_reader(reader, quality_base=None, colorspace=False, input_read=None):
         input_read: When file1 is a paired-end interleaved or SAM/BAM
             file, this specifies whether to only use the first or second read
             (1 or 2) or to use both reads (None).
+        alphabet: An Alphabet instance - the alphabet to use to validate
+            sequences.
     """
     if colorspace:
-        wrapped = SraColorspaceSequenceReader(reader, quality_base)
+        wrapped = SraColorspaceSequenceReader(
+            reader, quality_base=quality_base, alphabet=alphabet)
     else:
-        wrapped = SraSequenceReader(reader, quality_base)
-    
+        wrapped = SraSequenceReader(
+            reader, quality_base=quality_base, alphabet=alphabet)
+
     if not reader.paired or input_read == PAIRED:
         return wrapped
-    
+
     if input_read == READ1:
         return paired_to_read1(wrapped)
     else:
@@ -910,12 +957,12 @@ def sra_reader(reader, quality_base=None, colorspace=False, input_read=None):
 
 def guess_format_from_name(path, raise_on_failure=False):
     """Detect file format based on the file name.
-    
+
     Args:
         path: The filename to guess.
         raise_on_failure: Whether to raise an exception if the filename cannot
             be detected.
-    
+
     Returns:
         The format name.
     """
@@ -924,7 +971,7 @@ def guess_format_from_name(path, raise_on_failure=False):
         name = path
     elif hasattr(path, "name"):	 # seems to be an open file-like object
         name = path.name
-    
+
     if name:
         name, ext1, _ = splitext_compressed(name)
         ext = ext1.lower()
@@ -935,7 +982,7 @@ def guess_format_from_name(path, raise_on_failure=False):
             return 'fastq'
         elif ext in ('.sam', '.bam'):
             return ext[1:]
-    
+
     if raise_on_failure:
         raise UnknownFileType(
             "Could not determine whether file {0!r} is FASTA or FASTQ: file "
@@ -943,7 +990,7 @@ def guess_format_from_name(path, raise_on_failure=False):
 
 def create_seq_formatter(file1, file2=None, interleaved=False, **kwargs):
     """Create a formatter, deriving the format name from the file extension.
-    
+
     Args:
         file1, file2: Output files.
         interleaved: Whether the output should be interleaved (file2 must be
@@ -962,18 +1009,18 @@ def get_format(
         path, file_format=None, colorspace=False, qualities=None,
         line_length=None):
     """Create a SequenceFileFormat instance.
-    
+
     Args:
         path: The filename.
-        
+
         file_format: If set to None, file format is autodetected from the file
         name extension. Set to 'fasta', 'fastq', or 'sra-fastq' to not
         auto-detect. Colorspace is not auto-detected and must always be
         requested explicitly.
-        
+
         colorspace: If True, instances of the Colorspace... formats are
             returned.
-        
+
         qualities: When file_format is None, this can be set to True or False to
             specify whether the written sequences will have quality values.
             This is is used in two ways:
@@ -982,14 +1029,14 @@ def get_format(
               appropriately.
             * When False (no qualities available), an exception is raised when
               the auto-detected output format is FASTQ.
-    
+
     Returns:
         A SequenceFileFormat object.
     """
     if file_format is None:
         file_format = guess_format_from_name(
             path, raise_on_failure=qualities is None)
-    
+
     if file_format is None:
         if qualities is True:
             # Format not recognized, but know we want to write reads with
@@ -998,15 +1045,15 @@ def get_format(
         elif qualities is False:
             # Same, but we know that we want to write reads without qualities.
             file_format = 'fasta'
-    
+
     if file_format is None:
         raise UnknownFileType("Could not determine file type.")
-    
+
     if file_format == 'fastq' and qualities is False:
         raise ValueError(
             "Output format cannot be FASTQ since no quality values are "
             "available.")
-    
+
     file_format = file_format.lower()
     if file_format == 'fasta':
         if colorspace:
